#ifndef _DEVICE_H
#define _DEVICE_H
/*
 * Copyright (c) 1999 Greg Haerr <greg@censoft.com>
 *
 * Low-level Screen, Mouse and Keyboard device driver API's
 */

/* Changeable limits */
#define	MAX_CLIPRECTS 	200			/* maximum clip rectangles */
#define	MAX_CURSOR_SIZE 16			/* maximum cursor x and y size*/
#define MAX_CHAR_HEIGHT	16			/* maximum text bitmap height*/
#define	COORD_MIN	((COORD) -32768)	/* minimum coordinate value */
#define	COORD_MAX	((COORD) 32767)		/* maximum coordinate value */

/* Drawing modes*/
#define	MODE_SET	0	/* draw pixels as given (default) */
#define	MODE_XOR	1	/* draw pixels using XOR */
#define	MODE_OR		2	/* draw pixels using OR */
#define	MODE_AND	3	/* draw pixels using AND */
#define MODE_MAX	3

/* Mouse button bits in BUTTON*/
#define LBUTTON		04
#define MBUTTON		02
#define RBUTTON		01

/* typedefs changable based on target system*/
typedef int		COORD;		/* device coordinates*/
typedef int		MODE;		/* drawing mode*/
typedef unsigned int	COLORVAL;	/* device-independent color value*/
typedef unsigned char	PIXELVAL;	/* pixel color value*/
typedef unsigned int	BUTTON;		/* mouse button mask*/
typedef unsigned int	MODIFIER;	/* keyboard modifier mask (CTRL/SHIFT)*/
typedef int		FONTID;		/* font number, 0=default font*/
typedef int		BOOL;		/* boolean value*/
typedef unsigned char 	UCHAR;		/* text buffer*/
typedef unsigned short	IMAGEBITS;	/* bitmap image unit size*/

/* IMAGEBITS macros*/
#define IMAGE_WORDS(x)		(((x)+15)/16)
#define	IMAGE_BITSPERIMAGE	(sizeof(IMAGEBITS) * 8)
#define	IMAGE_FIRSTBIT		((IMAGEBITS) 0x8000)
#define	IMAGE_NEXTBIT(m)	((IMAGEBITS) ((m) >> 1))
#define	IMAGE_TESTBIT(m)	((m) & IMAGE_FIRSTBIT)	  /* use with shiftbit*/
#define	IMAGE_SHIFTBIT(m)	((IMAGEBITS) ((m) << 1))  /* for testbit*/

/* GetScreenInfo structure*/
typedef struct {
	COORD 	 rows;		/* number of rows on screen */
	COORD 	 cols;		/* number of columns on screen */
	int 	 xdpcm;		/* dots/centimeter in x direction */
	int 	 ydpcm;		/* dots/centimeter in y direction */
	int	 ncolors;	/* hw number of colors supported*/
	PIXELVAL black;		/* hw black color */
	PIXELVAL white;		/* hw white color */
	int 	 fonts;		/* number of built-in fonts */
	BUTTON 	 buttons;	/* buttons which are implemented */
	MODIFIER modifiers;	/* modifiers which are implemented */
} SCREENINFO, *PSCREENINFO;

/* GetFontInfo structure*/
typedef struct {
	int 	font;		/* font number */
	int 	height;		/* height of font */
	int 	maxwidth;	/* maximum width of any char */
	int 	baseline;	/* baseline of font */
	BOOL	fixed;		/* TRUE if font is fixed width */
	UCHAR	widths[256];	/* table of character widths */
} FONTINFO, *PFONTINFO;

/* In-core proportional font structure*/
typedef struct {
	char *		name;		/* font name*/
	int		maxwidth;	/* max width in pixels*/
	int		height;		/* height in pixels*/
	int		firstchar;	/* first character in bitmap*/
	int		size;		/* font size in characters*/
	IMAGEBITS *	bits;		/* 16-bit right-padded bitmap data*/
	unsigned short *offset;		/* 256 offsets into bitmap data*/
	unsigned char *	width;		/* 256 character widths or NULL*/
					/* if non-proportional*/
} FONT, *PFONT;

/* Interface to Screen Device Driver*/
typedef struct _screendevice {
	int	(*Open)(struct _screendevice *);
	void	(*Close)(void);
	void	(*GetScreenInfo)(PSCREENINFO psi);
	void	(*DrawPixel)(COORD x,COORD y,PIXELVAL c);
	PIXELVAL(*ReadPixel)(COORD x,COORD y);
	void	(*DrawHorzLine)(COORD x,COORD x2,COORD y,PIXELVAL c);
	void	(*DrawVertLine)(COORD x,COORD y1,COORD y2,PIXELVAL c);
	void	(*FillRect)(COORD x,COORD y,COORD w,COORD h,PIXELVAL c);
	/*void	(*DrawText)(COORD x,COORD y,const UCHAR *str,int count,
			PIXELVAL fg, FONTID fontid);*/
	BOOL	(*GetFontInfo)(FONTID fontid,PFONTINFO pfontinfo);
	void	(*GetTextSize)(const UCHAR *str,int cc,COORD *retwd,
			COORD *retht,FONTID fontid);
	void	(*GetTextBits)(UCHAR ch,IMAGEBITS *retmap,COORD *retwd,
			COORD *retht,FONTID fontid);
	COORD	xres;		/* X screen res*/
	COORD	yres;		/* Y screen res*/
	int	ncolors;	/* # screen colors*/
} SCREENDEVICE;

/* Interface to Mouse Device Driver*/
typedef struct _mousedevice {
	int	(*Open)(struct _mousedevice *);
	void	(*Close)(void);
	BUTTON	(*GetButtonInfo)(void);
	void	(*GetDefaultAccel)(int *pscale,int *pthresh);
	int	(*Read)(COORD *dx,COORD *dy,BUTTON *bp);
	int	(*Poll)(void);		/* not required if have select()*/
} MOUSEDEVICE;

/* Interface to Keyboard Device Driver*/
typedef struct _kbddevice {
	int  (*Open)(struct _kbddevice *pkd);
	void (*Close)(void);
	void (*GetModifierInfo)(MODIFIER *modifiers);
	int  (*Read)(UCHAR *buf,MODIFIER *modifiers);
	int  (*Poll)(void);		/* not required if have select()*/
} KBDDEVICE;

/* Clip rectangle: drawing allowed if point within rectangle*/
typedef struct {
	COORD 	x;		/* x coordinate of top left corner */
	COORD 	y;		/* y coordinate of top left corner */
	COORD 	width;		/* width of rectangle */
	COORD 	height;		/* height of rectangle */
} CLIPRECT;

/* point structure for polygons*/
typedef struct {
	COORD x;		/* x coordinate*/
	COORD y;		/* y coordinate*/
} POINT;

/* Clip areas*/
#define CLIP_VISIBLE		0
#define CLIP_INVISIBLE		1
#define CLIP_PARTIAL		2

#define TRUE			(1)
#define FALSE			(0)

/* entry points*/

/* serv_draw.c*/
int	GdOpenScreen(void);
void	GdCloseScreen(void);
void	GdSetMode(MODE m);
void	GdSetUseBackground(BOOL flag);
void	GdSetForeground(COLORVAL fg);
void	GdSetBackground(COLORVAL bg);
PIXELVAL GdFindColor(COLORVAL c);
void	GdGetScreenInfo(PSCREENINFO psi);
BOOL	GdGetFontInfo(FONTID fontid, PFONTINFO pfontinfo);
void	GdPoint(COORD x, COORD y);
void	GdLine(COORD x1, COORD y1, COORD x2, COORD y2);
void	GdRect(COORD x, COORD y, COORD width, COORD height);
void	GdFillRect(COORD x, COORD y, COORD width, COORD height);
void	GdText(COORD x, COORD y, const UCHAR *str, int count);
void	GdBitmap(COORD x,COORD y,COORD width,COORD height,IMAGEBITS *imagebits);
void	GdEllipse(COORD x, COORD y, COORD rx, COORD ry);
void	GdFillEllipse(COORD x, COORD y, COORD rx, COORD ry);
void	GdPoly(int count, POINT *points);
void	GdFillPoly(int count, POINT *points);
void	GdReadArea(COORD x,COORD y,COORD width,COORD height,PIXELVAL *pixels);
void	GdArea(COORD x,COORD y,COORD width,COORD height,PIXELVAL *pixels);
void	GdCopyArea(COORD srcx,COORD srcy,COORD width,COORD height,COORD destx,
		COORD desty);
extern SCREENDEVICE scrdev;

/* serv_clip.c*/
void 	GdSetClipRects(int count,CLIPRECT *table);
BOOL	GdClipPoint(COORD x,COORD y);
int	GdClipArea(COORD x1, COORD y1, COORD x2, COORD y2);
extern COORD clipminx, clipminy, clipmaxx, clipmaxy;

/* serv_mouse.c*/
int	GdOpenMouse(void);
void	GdCloseMouse(void);
void	GdGetButtonInfo(BUTTON *buttons);
void	GdRestrictMouse(COORD newminx,COORD newminy,COORD newmaxx,
		COORD newmaxy);
void	GdSetAccelMouse(int newthresh, int newscale);
void	GdMoveMouse(COORD newx, COORD newy);
int	GdReadMouse(COORD *xp, COORD *yp, BUTTON *bp);
void	GdMoveCursor(COORD x, COORD y);
void	GdSetCursor(COORD width,COORD height,COLORVAL foreground,
		COLORVAL background,IMAGEBITS *fgbitmap,IMAGEBITS *bgbitmap);
int 	GdShowCursor(void);
int 	GdHideCursor(void);
void	GdCheckCursor(COORD x1,COORD y1,COORD x2,COORD y2);
void 	GdFixCursor(void);
extern MOUSEDEVICE mousedev;

/* serv_kbd.c*/
int  	GdOpenKeyboard(void);
void 	GdCloseKeyboard(void);
void 	GdGetModifierInfo(MODIFIER *modifiers);
int  	GdReadKeyboard(UCHAR *buf, MODIFIER *modifiers);
extern KBDDEVICE kbddev;

#endif /*_DEVICE_H*/
